# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
#-----------------------------------------------------------------------------
# $Header: /project/isv-agent/ACCLib/lib/vcs/Hasys.pm,v 1.15 2008/10/06 05:09:34 aatre Exp $
#
# Veritas Software Corporation
#
# $Author: aatre $
# $Date: 2008/10/06 05:09:34 $
# $Revision: 1.15 $
#
# Provides: package Hasys.
#
# Functionality: wrapper to hasys Functionality
#
# Authors: 011787
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
# Simple calling an formatting of the hasys commands  
#
#   Display(<systems>, <Attributes>, &<ResultsList>)
#
#   GetStateGivenSystem(<SystemName>, &<SystemState>)
#
#   GetSystemStateList(&<ResultsList>)
#
# All commnds return 1 on sucess and undef on failure in line with perl booleans
#
# Not all hares functionality is currently available through this wrapper (see below)
#
#-----------------------------------------------------------------------------

#------------------
# version calls
#------------------
use Version;
Version::TellVersion("\$Header: /project/isv-agent/ACCLib/lib/vcs/Hasys.pm,v 1.15 2008/10/06 05:09:34 aatre Exp $::ETVHeader");
#-----------------------
# end of version calls
#-----------------------

#-------------------------------
# Declare the package name.
#-------------------------------
package Hasys;

#------------------------------------------
# Turn on strict for subs, vars and refs.
#------------------------------------------
use strict;

#---------------------------------------
# Include the required packages here.
#---------------------------------------
use LogInt;

#-----------------------------------
# ha command for system information.
#-----------------------------------
#----------------------------------------------
# Support VAD/VCSOne/VCS 5.0.. Atre 30 Jan 2008
#----------------------------------------------
my $sHasysCmd = ( $ENV { CLUSTER_HOME } ?
                  $ENV { CLUSTER_HOME } : $ENV { VCS_HOME } ) . "/bin/hasys";

#--------------------------------------------------------------
#  options marked with '*' are available
#
#  hasys -add <sys>
#  hasys -delete <sys>
#  hasys -freeze [-persistent] [-evacuate] <sys>
#  hasys -unfreeze [-persistent] <sys>
# *hasys -display [<system(s)>] [-attribute <attribute(s)>]
#  hasys -force <sys>
#  hasys -load <sys> <value>
# *hasys -state [<sys>]
#  hasys -list [<conditionals>]
#  hasys -value <sys> <attr>
#  hasys -nodeid [<nodeid>]
#  hasys [-help [-modify|-list]]
#---------------------------------------------------------------


=pod

-----------------------------------------------------------------------------
Function: Boolean Display( IN String Systems, 
                           IN String Attributes, 
                           OUT ArrayOfStrings Results )

Description:
  Displays the state of the system in the cluster.

Parameters:
  Systems    : String containing the name of the systems in the cluster.
  Attributes : String containing the list attributes to be displayed.
  Results    : Array containing the output from the display.

Return Value:
  Boolean value from the set of {0,1} where
        0 - The command execution fails.
        1 - The command execution succeeds.
-----------------------------------------------------------------------------

=cut

sub Display {
	
  my ( $sSystems, 
       $sAttributes, 
       $rlsResults ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10311,
                   "Hasys",
                   "Display", 
                   "Subroutine %s called with: System [%s], Attributes [%s]",
                   $sMyName, $sSystems, $sAttributes );

  #-------------------------------------------------------------------
  # some, all, or none of the above may be populated
  # results of the command are passed back in the $rlsResults param
  # hasys -display [<system(s)>] [-attribute <attribute(s)>]
  #-------------------------------------------------------------------

  #--------------------------------------------------------------------------
  # go through the passed in params building up the arg list for hasys call
  #
  # start with the generic display call
  #--------------------------------------------------------------------------
  my $sHasysDisplayCmd = "$sHasysCmd -display";

  if($sSystems)    { $sHasysDisplayCmd .= " $sSystems"; }
  if($sAttributes) { $sHasysDisplayCmd .= " -attribute $sAttributes"; }

  LogInt::Message( LogInt::LLTrace,
                   10312, 
                   "Hasys",
                   "Display",
                   "Going to call [%s]", $sHasysDisplayCmd);

  #------------------
  # call the command
  #------------------
  my @lsHasysOut = `$sHasysDisplayCmd 2>&1`;
  my $iReturnCode = $? >> 8;

  #--------------------------------------------------------
  # check the return code if not 0 then print an error 
  # message using the returned data
  #--------------------------------------------------------
  if ($iReturnCode != 0) {
    my $sHasysOut;
    if(@lsHasysOut > 1) {
      $sHasysOut = join('\n', @lsHasysOut);
    }
    else {
      $sHasysOut = $lsHasysOut[0];
    }
    chomp $sHasysOut;
    LogInt::Message( LogInt::LLError,
                     10313, 
                     "Hasys", 
                     "Display",
                     "Failed with message [%s]", $sHasysOut);
		      
    return 0; #error return
  }

  #-------------------------------------------------
  # assign the output to the passed in referance
  # #Group    Attribute           System    Value
  # fsapvol1  AutoFailOver        global    1
  # etc..
  #-------------------------------------------------

  for my $sEachHasysOut (@lsHasysOut) {
    if($sEachHasysOut =~ /^#/) {
      #ignore lines starting with '#'
    }
    else {
      chomp $sEachHasysOut;
      $sEachHasysOut =~ s/\s+/ /g; #remove multiple spaces
      LogInt::Message( LogInt::LLTraceTrace,
                       10314,
                       "Hasys",
                       "Display",
                       "Adding [%s] to rlsResults", $sEachHasysOut);
		      
      push(@$rlsResults, $sEachHasysOut);
    }
  }

  my $iEntryCount = @$rlsResults;
  LogInt::Message( LogInt::LLTrace,
                   10315,
                   "Hasys",
                   "Display",
                   "Found [%s] entries", $iEntryCount);

  return 1;
}


=pod

-----------------------------------------------------------------------------
Function: Boolean GetStateGivenSystem( IN String System,
                                       OUT String State )

Description:
  Get the state of the given system/node in the cluster.

Variables:
  System : Name of the system.
  State  : State of the system.

Return Value:
  Boolean value from the set of {0,1} where
        0 - The command execution fails.
        1 - The command execution succeeds.
-----------------------------------------------------------------------------

=cut

sub GetStateGivenSystem {
	
  my ( $sSystem, 
       $rsState ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10316,
                   "Hasys",
                   "GetStateGivenSystem",
                   "Subroutine %s called with: System [%s]",
                   $sMyName, $sSystem );

  #--------------------------
  # hasys -state [<sys>]
  #--------------------------

  #-------------------------------
  # make the hasys command string
  #-------------------------------
  my $sHasysStateCmd = "$sHasysCmd -state " . $sSystem;

  LogInt::Message( LogInt::LLTrace,
                   10317,
                   "Hasys",
                   "GetStateGivenSystem",
                   "Going to call [%s]", $sHasysStateCmd );

  #-------------------
  # call the command
  #-------------------
  my @lsHasysOut = `$sHasysStateCmd 2>&1`;
  my $iReturnCode = $? >> 8;

  #----------------------------------------------------
  # check the return code if not 0 then print an error 
  # message using the returned data
  #----------------------------------------------------
  if ($iReturnCode != 0) {
    my $sHasysOut;
    if(@lsHasysOut > 1) {
      $sHasysOut = join('\n', @lsHasysOut);
    }
    else {
      $sHasysOut = $lsHasysOut[0];
    }
    chomp $sHasysOut;
    LogInt::Message( LogInt::LLError,
                     10318,
                     "Hasys",
                     "GetStateGivenSystem",
                     "Failed with message [%s]", $sHasysOut);
			
    return 0; #error return
  }

  my $sSystemState = $lsHasysOut[0];
  chomp $sSystemState; #remove any newline
  LogInt::Message( LogInt::LLTrace,
                   10319, 
                   "Hasys", 
                   "GetStateGivenSystem",
                   "Got state [%s] for system [%s]", $sSystemState, $sSystem );

  #--------------------------------------
  # set up the results to be passed back
  #--------------------------------------
  $$rsState = $sSystemState;

  return 1;
}


=pod

-----------------------------------------------------------------------------
Function: Boolean GetSystemStateList( OUT ArrayOfStrings Results )

Description:
  Gets the state of the each system/node in the cluster.

Variables:
  Results : Array containing each system state. 
            The format of the output will be like
		#System   Attribute State
                gps-lin5 SysState RUNNING 

Return Value:
  Boolean value from the set of {0,1} where
          0 - The command execution fails.
          1 - The command execution succeeds.
-----------------------------------------------------------------------------

=cut

sub GetSystemStateList {

  my ( $rlsResults ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace, 
                   10320, 
                   "Hasys",
                   "GetSystemStateList",
                   "Subroutine %s called", $sMyName );

  #-------------------------
  # hasys -state [<sys>]
  #-------------------------
  #--------------------------------
  # make the hasys command string
  #--------------------------------
  my $sHasysStateCmd = $sHasysCmd . " -state";
  LogInt::Message( LogInt::LLTrace,
                   10321,
                   "Hasys", 
                   "GetSystemStateList",
                   "Going to call [%s]", $sHasysStateCmd );
  #-----------------------
  # call the command
  #-----------------------
  my @lsHasysOut = `$sHasysStateCmd 2>&1`;
  my $iReturnCode = $? >> 8;

  #---------------------------------------------------
  # check the return code if not 0 then print an error
  # message using the returned data
  #---------------------------------------------------
  if ($iReturnCode != 0) {
    my $sHasysOut = undef;
    if(@lsHasysOut > 1) {
      $sHasysOut = join('\n', @lsHasysOut);
    } 
    else {
      $sHasysOut = $lsHasysOut[0];
    }
    chomp $sHasysOut;
    LogInt::Message( LogInt::LLError,
                     10322,
                     "Hasys",
                     "GetSystemStateList",
                     "Failed with message [%s]", $sHasysOut );
    return 0; #error return
  }

  #-----------------------------------------------
  # assign the output to the passed in referance
  #
  # #System      Attribute         Value
  # helium       SysState          RUNNING
  # hydrogen     SysState          RUNNING
  #-----------------------------------------------


  for my $sEachHasysOut (@lsHasysOut) {
    if($sEachHasysOut =~ /^#/) {
      #ignore lines starting with '#'
    } 
    else {
      chomp $sEachHasysOut; #remove any newline
      $sEachHasysOut =~ s/\s+/ /g; #remove multiple spaces
      LogInt::Message( LogInt::LLTraceTrace, 
                       10323,
                       "Hasys", 
                       "GetSystemStateList",
                       "Adding [%s] to rlsResults", $sEachHasysOut );
			  
      push(@$rlsResults, $sEachHasysOut);
    }
  }

  my $iEntryCount = @$rlsResults;

  LogInt::Message( LogInt::LLTrace,
                   10324,
                   "Hasys",
                   "GetSystemStateList",
                   "Found [%s] entries", $iEntryCount );
		       
  return 1;
}


1;

