# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
#-----------------------------------------------------------------------------
# $Header: /project/isv-agent/ACCLib/lib/vcs/VCSDefines.pm,v 1.17 2008/10/06 05:09:35 aatre Exp $
#
# Veritas Software Corporation
#
# $Author: aatre $
# $Date: 2008/10/06 05:09:35 $
# $Revision: 1.17 $
#
# Provides: Number of small packages (see below)
#
# Functionality: defines the numbers which the VCS engine and scripts can use
#
# Authors: 011787
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
# The following constant definitions for use in scripts
#
#   VCSDef_CleanReason
#   VCSDef_MonitorExit
#   VCSDef_CleanExit
#   VCSDef_ResourceState
#   VCSDef_ResourceIState
#
#-----------------------------------------------------------------------------

#-----------------
# version calls
#-----------------
use Version;
Version::TellVersion("\$Header: /project/isv-agent/ACCLib/lib/vcs/VCSDefines.pm,v 1.17 2008/10/06 05:09:35 aatre Exp $::ETVHeader");
#----------------------
# end of version calls
#----------------------

#----------------------------------
# for internationalized logging
#----------------------------------
use LogInt; 

#-------------------------------------------
# Turn on strict for subs, refs and vars.
#-------------------------------------------
use strict;

#------------------------------------------------------------------
# Declare the max size of the message allowed to log to the engine.
#------------------------------------------------------------------

#---------------------
# Clean reason codes
#---------------------

package VCSDef_CleanReason;

use constant OFFLINE_HUNG                  => 0;
use constant OFFLINE_INEFFECTIVE           => 1;
use constant ONLINE_HUNG                   => 2;
use constant ONLINE_INEFFECTIVE            => 3;
use constant UNEXPECTED_OFFLINE            => 4;
use constant MONITOR_HUNG                  => 5;


=pod

-----------------------------------------------------------------------------
Function: Boolean IsValid( IN Integer Value )

Description:
  Checks given value is with in the boundary values.

Parameters:
  Value : Value needs to be checked.

Return Value:
  Boolean value in the set of {0,1} where
          0 - false
          1 - true
-----------------------------------------------------------------------------

=cut

sub IsValid {

  my ( $iValueToCheck ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace, 
  10325, 
  "VCSDefines", 
  "VCSDef_CleanReason",
  "Subroutine <%s> called with:
  Value          [%s]",
  $sMyName, $iValueToCheck );

  #----------------------------------------------------
  # we check it's actually a number we've been handed.
  #----------------------------------------------------
  if($iValueToCheck =~ /^[0-9]+$/) {
    if(($iValueToCheck >= VCSDef_CleanReason::OFFLINE_HUNG) &&
       ($iValueToCheck <= VCSDef_CleanReason::MONITOR_HUNG)) {
      LogInt::Message( LogInt::LLTrace, 
                       10326, 
                       "VCSDefines", 
                       "VCSDef_CleanReason",
                       "Checking IsValid, got passed [%s] returning OK", 
                       $iValueToCheck );
      return 1;
    }
  }

  #-------------------------------------------------------------
  # if we get here then it's invalid (only use level 'trace' - 
  # since we are being called to check it only).
  #-------------------------------------------------------------
  LogInt::Message( LogInt::LLTrace, 
                   10327, 
                   "VCSDefines", 
                   "VCSDef_CleanReason",
                   "Checking IsValid, got passed [%s] which is out of the defined range",
                   $iValueToCheck );
  return 0; #invalid value
}
 

=pod

-----------------------------------------------------------------------------
Function: Boolean GetString( IN Integer Value,
                             OUT String String )

Description:
  Gives the corresponding string to the given value.

Parameters:
  Value  : An integer value.
  String : String corresponding to the given value.

Return Value:
  Boolean value in the set of {0,1} where
          0 - false
          1 - true
-----------------------------------------------------------------------------

=cut

sub GetString {

  my ( $iValue, 
       $rsString ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10328, 
  "VCSDefines",
  "VCSDef_CleanReason",
  "Subroutine <%s> called with:
  Value         [%s]",
  $sMyName, $iValue );
 
  my %shValueToString = ( VCSDef_CleanReason::OFFLINE_HUNG              => "Offline Hung",
                          VCSDef_CleanReason::OFFLINE_INEFFECTIVE       => "Offline Ineffective",
                          VCSDef_CleanReason::ONLINE_HUNG               => "Online Hung",
                          VCSDef_CleanReason::ONLINE_INEFFECTIVE        => "Online Ineffective",
                          VCSDef_CleanReason::UNEXPECTED_OFFLINE        => "Unexpected Offline",
                          VCSDef_CleanReason::MONITOR_HUNG              => "Monitor Hung",
                        );

  if(exists $shValueToString{$iValue}) {
    $$rsString = $shValueToString{$iValue};
    return 1;
  } 
  else {
    LogInt::Message( LogInt::LLError,
                     10329, 
                     "VCSDefines",
                     "VCSDef_CleanReason",
                     "Couldn't find corresponding string for [%s]",
                     $iValue );
    #-----------------------------------------------------------------------
    # give the string a value to avoid "uninitialized value" errors....
    #-----------------------------------------------------------------------
    $$rsString = "UNKNOWN"; 
    return 0; # could not find valid string value for presented iValue...
  }
}


1; #end VCSDef_CleanReason


#--------------------------
# Monitor exit codes.
#--------------------------
package VCSDef_MonitorExit;

use constant UNKNOWN         => 99;
use constant ONLINE          => 110;
use constant PARTIAL_ONLINE  => 105;
use constant OFFLINE         => 100;
use constant TIMEOUT         => 999999;

1; #end VCSDef_MonitorExit


#----------------------
# Clean exit Codes
#----------------------
package VCSDef_CleanExit;

use constant PASSED => 0;
use constant FAILED => 1;

1; #end VCSDef_CleanExit


#------------------------
# Resource State codes
#------------------------
package VCSDef_ResourceState;

use constant OFFLINE                   => 0;
use constant FAULTED                   => 1;
use constant ONLINE                    => 2;
use constant ONLINE_STATE_UNKNOWN      => 3; #not sure this is correct value!
use constant ONLINE_MONITOR_TIMEDOUT   => 4; #not sure this is correct value!
use constant ONLINE_UNABLE_TO_OFFLINE  => 5; #not sure this is correct value!
use constant OFFLINE_STATE_UNKNOWN     => 6; #not sure this is correct value!
use constant FAULTED_RESTARTING        => 7; #not sure this is correct value!


=pod

-----------------------------------------------------------------------------
Function: Boolean IsValid( IN Integer Value )

Description:
  Checks given value is with in the boundary values.

Parameters:
  Value : Value needs to be checked.

Return Value:
  Boolean value in the set of {0,1} where
          0 - false
          1 - true
-----------------------------------------------------------------------------

=cut

sub IsValid {

  my ( $iValueToCheck ) = @_;
  
  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10330,
  "VCSDefines",
  "VCSDef_ResourceState",
  "Subroutine <%s> called with:
  Value        [%s]",
  $sMyName, $iValueToCheck );

  #-----------------------------------------------------
  # we check it's actually a number we've been handed
  #-----------------------------------------------------
  if($iValueToCheck =~ /^[0-9]+$/) {
    if(($iValueToCheck >= VCSDef_ResourceState::OFFLINE) &&
       ($iValueToCheck <= VCSDef_ResourceState::FAULTED_RESTARTING)) {
      LogInt::Message( LogInt::LLTrace,
                       10331,
                       "VCSDefines",
                       "VCSDef_ResourceState",
                       "Checking IsValid, got passed [%s] returning OK",
                       $iValueToCheck );
      return 1;
    }
  }

  #-------------------------------------------------------------------
  # if we get here then it's invalid (only use level 'trace' - since 
  # we are being called to check it only)
  #-------------------------------------------------------------------
  LogInt::Message( LogInt::LLTrace, 
                   10332, 
                   "VCSDefines", 
                   "VCSDef_ResourceState",
                   "Checking IsValid, got passed [%s] which is out of the defined range",
                   $iValueToCheck );

  return 0; #invalid value
}


=pod

-----------------------------------------------------------------------------
Function: Boolean GetString( IN Integer Value,
                             OUT String String )

Description:
  Gives the corresponding string to the given value.

Parameters:
  Value  : An integer value.
  String : String corresponding to the given value.

Return Value:
  Boolean value in the set of {0,1} where
          0 - false
          1 - true
-----------------------------------------------------------------------------

=cut

sub GetString {

  my ( $iValue, 
       $rsString ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10333, 
  "VCSDefines",
  "VCSDef_ResourceState",
  "Subroutine <%s> called with:
  Value           [%s]",
  $sMyName, $iValue );

  my %shValueToString = (
    VCSDef_ResourceState::OFFLINE                    => "Offline",
    VCSDef_ResourceState::FAULTED                    => "Faulted",
    VCSDef_ResourceState::ONLINE                     => "Online",
    VCSDef_ResourceState::ONLINE_STATE_UNKNOWN       => "Online | State Unknown",
    VCSDef_ResourceState::ONLINE_MONITOR_TIMEDOUT    => "Online | Monitor Timedout",
    VCSDef_ResourceState::ONLINE_UNABLE_TO_OFFLINE   => "Online | Unable To Offline",
    VCSDef_ResourceState::OFFLINE_STATE_UNKNOWN      => "Offline | State Unknown",
    VCSDef_ResourceState::FAULTED_RESTARTING         => "Failed | Restarting" );

  if(exists $shValueToString{$iValue}) {
    $$rsString = $shValueToString{$iValue};
    return 1;

  } 
  else {
    LogInt::Message( LogInt::LLError,
                     10334, 
                     "VCSDefines",
                     "VCSDef_ResourceState",
                     "Couldn't find corresponding string for [%s]", 
                     $iValue );

    #--------------------------------------------------------------------
    # give the string a value to avoid "uninitialized value" errors....
    #--------------------------------------------------------------------
    $$rsString = "UNKNOWN"; 
    return 0; # could not find valid string value for presented iValue...
  }
}


1; #end VCSDef_ResourceState


#--------------------------
# Resource IState codes.
#--------------------------
package VCSDef_ResourceIState;

use constant NOT_WAITING                                  => 0;
use constant WAITING_TO_GO_ONLINE                         => 1;
use constant WAITING_FOR_CHILDREN_ONLINE                  => 2;
use constant WAITING_TO_GO_OFFLINE                        => 3;
use constant WAITING_TO_GO_OFFLINE_PROPAGATE              => 4;
use constant WAITING_TO_GO_OFFLINE_PATH                   => 5;
use constant WAITING_TO_GO_OFFLINE_REVERSE                => 6;
use constant WAITING_TO_GO_ONLINE_REVERSE                 => 7;
use constant WAITING_TO_GO_ONLINE_REVERSE_PROPAGATE       => 8;
use constant WAITING_TO_GO_ONLINE_REVERSE_PATH            => 9;


=pod

-----------------------------------------------------------------------------
Function: Boolean IsValid( IN Integer Value )

Description:
  Checks given value is with in the boundary values.

Parameters:
  Value : Value needs to be checked.

Return Value:
  Boolean value in the set of {0,1} where
          0 - false
          1 - true
-----------------------------------------------------------------------------

=cut

sub IsValid {

  my ( $iValueToCheck ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10335,
  "VCSDefines", 
  "VCSDef_ResourceIState",
  "Subroutine <%s> called with:
  Value          [%s]",
  $sMyName, $iValueToCheck );

  #------------------------------------------------------
  # we check it's actually a number we've been handed
  #------------------------------------------------------
  if($iValueToCheck =~ /^[0-9]+$/) {
    if(($iValueToCheck >= VCSDef_ResourceIState::NOT_WAITING) &&
       ($iValueToCheck <= VCSDef_ResourceIState::WAITING_TO_GO_ONLINE_REVERSE_PATH)) {
      LogInt::Message( LogInt::LLTrace,
                       10336,
                       "VCSDefines", 
                       "VCSDef_ResourceIState",
                       "Checking IsValid, got passed [%s] returning OK",
                       $iValueToCheck );
      return 1;
    }
  }

  #------------------------------------------------------------------
  # if we get here then it's invalid (only use level 'trace' - since 
  # we are being called to check it only)
  #------------------------------------------------------------------
  LogInt::Message( LogInt::LLTrace,
                       10337,
                       "VCSDefines",
                       "VCSDef_ResourceIState",
                       "Checking IsValid, got passed [%s] which is out of the defined range",
                       $iValueToCheck );
  return 0; #invalid value
}


=pod

-----------------------------------------------------------------------------
Function: Boolean GetString( IN Integer Value,
                             OUT String String )

Description:
  Gives the corresponding string to the given value.

Parameters:
  Value  : An integer value.
  String : String corresponding to the given value.

Return Value:
  Boolean value in the set of {0,1} where
          0 - false
          1 - true
-----------------------------------------------------------------------------

=cut

sub GetString {

  my ( $iValue,
       $rsString ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace, 
  10338,
  "VCSDefines", 
  "VCSDef_ResourceIState",
  "Subroutine <%s> called with:
  Value           [%s]",
  $sMyName, $iValue );

  my %shValueToString = (
    VCSDef_ResourceIState::NOT_WAITING                              => "Not Waiting",
    VCSDef_ResourceIState::WAITING_TO_GO_ONLINE                     => "Waiting To Go Online",
    VCSDef_ResourceIState::WAITING_FOR_CHILDREN_ONLINE              => "Waiting For Children Online",
    VCSDef_ResourceIState::WAITING_TO_GO_OFFLINE                    => "Waiting To Go Offline",
    VCSDef_ResourceIState::WAITING_TO_GO_OFFLINE_PROPAGATE          => "Waiting To Go Offline (propagate)",
    VCSDef_ResourceIState::WAITING_TO_GO_OFFLINE_PATH               => "Waiting To Go Offline (path)",
    VCSDef_ResourceIState::WAITING_TO_GO_OFFLINE_REVERSE            => "Waiting To Go Offline (reverse)",
    VCSDef_ResourceIState::WAITING_TO_GO_ONLINE_REVERSE             => "Waiting To Go Online (reverse)",
    VCSDef_ResourceIState::WAITING_TO_GO_ONLINE_REVERSE_PROPAGATE   => "Waiting To Go Online (reverse/propagate)",
    VCSDef_ResourceIState::WAITING_TO_GO_ONLINE_REVERSE_PATH        => "Waiting To Go Online (reverse/path)"
  );

  if(exists $shValueToString{$iValue}) {
    $$rsString = $shValueToString{$iValue};
    return 1; # found valid string value for presented iValue...
  }
  else {
    LogInt::Message( LogInt::LLError, 
                     10339,
                     "VCSDefines", 
                     "VCSDef_ResourceIState",
                     "Couldn't find corresponding string for [%s]",
                     $iValue );
    #----------------------------------------------------------------------
    # give the string a value to avoid "uninitialized value" errors....
    #----------------------------------------------------------------------
    $$rsString = "UNKNOWN";
    return 0; # could not find valid string value for presented iValue...
  }
}


1; #end VCSDef_ResourceIState


